# Copyright (c) 2024 Phase Advanced Sensor Systems, Inc.
from ..device import Reg32, Reg32W, AReg32
from ..stm32 import flash_type1


class FLASH(flash_type1.FLASH):
    '''
    Driver for the FLASH device on the STM32L4 series of MCUs.
    '''
    REGS = [AReg32('ACR',           0x000, [('LATENCY',             0,  2),
                                            ('PRFTEN',              8),
                                            ('ICEN',                9),
                                            ('DCEN',                10),
                                            ('ICRST',               11),
                                            ('DCRST',               12),
                                            ('RUN_PD',              13),
                                            ('SLEEP_PD',            14),
                                            ]),
            Reg32W('PDKEYR',        0x004),
            Reg32W('KEYR',          0x008),
            Reg32W('OPTKEYR',       0x00C),
            AReg32('SR',            0x010, [('EOP',                 0),
                                            ('OPERR',               1),
                                            ('PROGERR',             3),
                                            ('WRPERR',              4),
                                            ('PGAERR',              5),
                                            ('SIZERR',              6),
                                            ('PGSERR',              7),
                                            ('MISSERR',             8),
                                            ('FASTERR',             9),
                                            ('RDERR',               14),
                                            ('OPTVERR',             15),
                                            ('BSY',                 16),
                                            ('PEMPTY',              17),
                                            ]),
            AReg32('CR',            0x014, [('PG',                  0),
                                            ('PER',                 1),
                                            ('MER1',                2),
                                            ('PNB',                 3,  10),
                                            ('STRT',                16),
                                            ('OPTSTR',              17),
                                            ('FSTPG',               18),
                                            ('EOPIE',               24),
                                            ('ERRIE',               25),
                                            ('RDERRIE',             26),
                                            ('OBL_LAUNCH',          27),
                                            ('OPTLOCK',             30),
                                            ('LOCK',                31),
                                            ]),
            AReg32('ECCR',          0x018, [('ADDR_ECC',            0, 18),
                                            ('SYSF_ECC',            20),
                                            ('ECCCIE',              24),
                                            ('ECCC',                30),
                                            ('ECCD',                31),
                                            ]),
            AReg32('OPTR',          0x020, [('RDP',                 0,  7),
                                            ('BOR_LEV',             8,  10),
                                            ('nRST_STOP',           12),
                                            ('nRST_STDBY',          13),
                                            ('nRST_SHDW',           14),
                                            ('IWDG_SW',             16),
                                            ('IWDG_STOP',           17),
                                            ('IWDG_STDBY',          18),
                                            ('WWDG_SW',             19),
                                            ('nBOOT1',              23),
                                            ('SRAM2_PE',            24),
                                            ('SRAM2_RST',           25),
                                            ('nSWBOOT0',            26),
                                            ('nBOOT0',              27),
                                            ]),
            Reg32 ('PCROP1SR',      0x024),
            Reg32 ('PCROP1ER',      0x028),
            Reg32 ('WRP1AR',        0x02C),
            Reg32 ('WRP1BR',        0x030),
            ]

    def __init__(self, target, ap, name, dev_base, mem_base, max_write_freq,
                 otp_base, otp_len, **kwargs):
        super().__init__(target, FLASH.REGS, 2048, ap, name, dev_base,
                         mem_base, max_write_freq, otp_base, otp_len, **kwargs)
